@@ -3,12 +3,11 @@ use crate::app::HelpMenuLine;
 use crate::app::NodeFilter;
 use crate::app::NodeSorter;
 use crate::app::NodeSorterApplicable;
-use crate::default_config;
 use crate::ui::Border;
 use crate::ui::Constraint;
 use crate::ui::Layout;
 use crate::ui::Style;
-use anyhow::Result;
+use indexmap::IndexMap;
 use indexmap::IndexSet;
 use serde::{Deserialize, Serialize};
 use std::collections::BTreeMap;
@@ -18,10 +17,10 @@ use std::collections::HashMap;
 #[serde(deny_unknown_fields)]
 pub struct Action {
     #[serde(default)]
-    help: Option<String>,
+    pub help: Option<String>,
 
     #[serde(default)]
-    messages: Vec<ExternalMsg>,
+    pub messages: Vec<ExternalMsg>,
 }
 
 impl Action {
@@ -39,12 +38,6 @@ impl Action {
         }
     }
 
-    pub fn extend(mut self, other: Self) -> Self {
-        self.help = other.help.or(self.help);
-        self.messages = other.messages;
-        self
-    }
-
     /// Get a reference to the action's help.
     pub fn help(&self) -> &Option<String> {
         &self.help
@@ -60,19 +53,13 @@ impl Action {
 #[serde(deny_unknown_fields)]
 pub struct NodeTypeConfig {
     #[serde(default)]
-    style: Style,
+    pub style: Style,
 
     #[serde(default)]
-    meta: HashMap<String, String>,
+    pub meta: HashMap<String, String>,
 }
 
 impl NodeTypeConfig {
-    pub fn extend(mut self, other: Self) -> Self {
-        self.style = self.style.extend(other.style);
-        self.meta.extend(other.meta);
-        self
-    }
-
     /// Get a reference to the node type config's style.
     pub fn style(&self) -> &Style {
         &self.style
@@ -88,35 +75,25 @@ impl NodeTypeConfig {
 #[serde(deny_unknown_fields)]
 pub struct NodeTypesConfig {
     #[serde(default)]
-    directory: NodeTypeConfig,
+    pub directory: NodeTypeConfig,
 
     #[serde(default)]
-    file: NodeTypeConfig,
+    pub file: NodeTypeConfig,
 
     #[serde(default)]
-    symlink: NodeTypeConfig,
+    pub symlink: NodeTypeConfig,
 
     #[serde(default)]
-    mime_essence: HashMap<String, NodeTypeConfig>,
+    pub mime_essence: HashMap<String, NodeTypeConfig>,
 
     #[serde(default)]
-    extension: HashMap<String, NodeTypeConfig>,
+    pub extension: HashMap<String, NodeTypeConfig>,
 
     #[serde(default)]
-    special: HashMap<String, NodeTypeConfig>,
+    pub special: HashMap<String, NodeTypeConfig>,
 }
 
 impl NodeTypesConfig {
-    fn extend(mut self, other: Self) -> Self {
-        self.directory = self.directory.extend(other.directory);
-        self.file = self.file.extend(other.file);
-        self.symlink = self.symlink.extend(other.symlink);
-        self.mime_essence.extend(other.mime_essence);
-        self.extension.extend(other.extension);
-        self.special.extend(other.special);
-        self
-    }
-
     /// Get a reference to the node types config's directory.
     pub fn directory(&self) -> &NodeTypeConfig {
         &self.directory
@@ -152,23 +129,16 @@ impl NodeTypesConfig {
 #[serde(deny_unknown_fields)]
 pub struct UiConfig {
     #[serde(default)]
-    pub(crate) prefix: Option<String>,
+    pub prefix: Option<String>,
 
     #[serde(default)]
-    pub(crate) suffix: Option<String>,
+    pub suffix: Option<String>,
 
     #[serde(default)]
-    pub(crate) style: Style,
+    pub style: Style,
 }
 
 impl UiConfig {
-    pub fn extend(mut self, other: Self) -> Self {
-        self.prefix = other.prefix.or(self.prefix);
-        self.suffix = other.suffix.or(self.suffix);
-        self.style = self.style.extend(other.style);
-        self
-    }
-
     /// Get a reference to the ui config's prefix.
     pub fn prefix(&self) -> &Option<String> {
         &self.prefix
@@ -189,10 +159,10 @@ impl UiConfig {
 #[serde(deny_unknown_fields)]
 pub struct UiElement {
     #[serde(default)]
-    format: Option<String>,
+    pub format: Option<String>,
 
     #[serde(default)]
-    style: Style,
+    pub style: Style,
 }
 
 impl UiElement {
@@ -217,23 +187,16 @@ impl UiElement {
 #[serde(deny_unknown_fields)]
 pub struct TableRowConfig {
     #[serde(default)]
-    cols: Option<Vec<UiElement>>,
+    pub cols: Option<Vec<UiElement>>,
 
     #[serde(default)]
-    style: Style,
+    pub style: Style,
 
     #[serde(default)]
-    height: Option<u16>,
+    pub height: Option<u16>,
 }
 
 impl TableRowConfig {
-    fn extend(mut self, other: Self) -> Self {
-        self.cols = other.cols.or(self.cols);
-        self.style = self.style.extend(other.style);
-        self.height = other.height.or(self.height);
-        self
-    }
-
     /// Get a reference to the table row config's cols.
     pub fn cols(&self) -> &Option<Vec<UiElement>> {
         &self.cols
@@ -254,35 +217,25 @@ impl TableRowConfig {
 #[serde(deny_unknown_fields)]
 pub struct TableConfig {
     #[serde(default)]
-    header: TableRowConfig,
+    pub header: TableRowConfig,
 
     #[serde(default)]
-    row: TableRowConfig,
+    pub row: TableRowConfig,
 
     #[serde(default)]
-    style: Style,
+    pub style: Style,
 
     #[serde(default)]
-    tree: Option<(UiElement, UiElement, UiElement)>,
+    pub tree: Option<(UiElement, UiElement, UiElement)>,
 
     #[serde(default)]
-    col_spacing: Option<u16>,
+    pub col_spacing: Option<u16>,
 
     #[serde(default)]
-    col_widths: Option<Vec<Constraint>>,
+    pub col_widths: Option<Vec<Constraint>>,
 }
 
 impl TableConfig {
-    pub fn extend(mut self, other: Self) -> Self {
-        self.header = self.header.extend(other.header);
-        self.row = self.row.extend(other.row);
-        self.style = self.style.extend(other.style);
-        self.tree = other.tree.or(self.tree);
-        self.col_spacing = other.col_spacing.or(self.col_spacing);
-        self.col_widths = other.col_widths.or(self.col_widths);
-        self
-    }
-
     /// Get a reference to the table config's header.
     pub fn header(&self) -> &TableRowConfig {
         &self.header
@@ -318,23 +271,19 @@ impl TableConfig {
 #[serde(deny_unknown_fields)]
 pub struct LogsConfig {
     #[serde(default)]
-    info: UiElement,
+    pub info: UiElement,
 
     #[serde(default)]
-    success: UiElement,
+    pub success: UiElement,
 
     #[serde(default)]
-    error: UiElement,
+    pub warning: UiElement,
+
+    #[serde(default)]
+    pub error: UiElement,
 }
 
 impl LogsConfig {
-    pub fn extend(mut self, other: Self) -> Self {
-        self.info = self.info.extend(other.info);
-        self.success = self.success.extend(other.success);
-        self.error = self.error.extend(other.error);
-        self
-    }
-
     /// Get a reference to the logs config's info.
     pub fn info(&self) -> &UiElement {
         &self.info
@@ -349,25 +298,24 @@ impl LogsConfig {
     pub fn error(&self) -> &UiElement {
         &self.error
     }
+
+    /// Get a reference to the logs config's warning.
+    pub fn warning(&self) -> &UiElement {
+        &self.warning
+    }
 }
 
 #[derive(Debug, Clone, Default, Serialize, Deserialize)]
 #[serde(deny_unknown_fields)]
 pub struct SortDirectionIdentifiersUi {
     #[serde(default)]
-    forward: UiElement,
+    pub forward: UiElement,
 
     #[serde(default)]
-    reverse: UiElement,
+    pub reverse: UiElement,
 }
 
 impl SortDirectionIdentifiersUi {
-    pub fn extend(mut self, other: Self) -> Self {
-        self.forward = self.forward.extend(other.forward);
-        self.reverse = self.reverse.extend(other.reverse);
-        self
-    }
-
     /// Get a reference to the sort direction identifiers ui's forward.
     pub fn forward(&self) -> &UiElement {
         &self.forward
@@ -383,33 +331,22 @@ impl SortDirectionIdentifiersUi {
 #[serde(deny_unknown_fields)]
 pub struct SortAndFilterUi {
     #[serde(default)]
-    separator: UiElement,
+    pub separator: UiElement,
 
     #[serde(default)]
-    default_identifier: UiElement,
+    pub default_identifier: UiElement,
 
     #[serde(default)]
-    sort_direction_identifiers: SortDirectionIdentifiersUi,
+    pub sort_direction_identifiers: SortDirectionIdentifiersUi,
 
     #[serde(default)]
-    sorter_identifiers: HashMap<NodeSorter, UiElement>,
+    pub sorter_identifiers: HashMap<NodeSorter, UiElement>,
 
     #[serde(default)]
-    filter_identifiers: HashMap<NodeFilter, UiElement>,
+    pub filter_identifiers: HashMap<NodeFilter, UiElement>,
 }
 
 impl SortAndFilterUi {
-    pub fn extend(mut self, other: Self) -> Self {
-        self.separator = self.separator.extend(other.separator);
-        self.default_identifier = self.default_identifier.extend(other.default_identifier);
-        self.sort_direction_identifiers = self
-            .sort_direction_identifiers
-            .extend(other.sort_direction_identifiers);
-        self.sorter_identifiers.extend(other.sorter_identifiers);
-        self.filter_identifiers.extend(other.filter_identifiers);
-        self
-    }
-
     /// Get a reference to the sort and filter ui's separator.
     pub fn separator(&self) -> &UiElement {
         &self.separator
@@ -440,35 +377,25 @@ impl SortAndFilterUi {
 #[serde(deny_unknown_fields)]
 pub struct PanelUi {
     #[serde(default)]
-    default: PanelUiConfig,
+    pub default: PanelUiConfig,
 
     #[serde(default)]
-    table: PanelUiConfig,
+    pub table: PanelUiConfig,
 
     #[serde(default)]
-    sort_and_filter: PanelUiConfig,
+    pub sort_and_filter: PanelUiConfig,
 
     #[serde(default)]
-    selection: PanelUiConfig,
+    pub selection: PanelUiConfig,
 
     #[serde(default)]
-    input_and_logs: PanelUiConfig,
+    pub input_and_logs: PanelUiConfig,
 
     #[serde(default)]
-    help_menu: PanelUiConfig,
+    pub help_menu: PanelUiConfig,
 }
 
 impl PanelUi {
-    fn extend(mut self, other: Self) -> Self {
-        self.default = self.default.extend(other.default);
-        self.table = self.table.extend(other.table);
-        self.sort_and_filter = self.sort_and_filter.extend(other.sort_and_filter);
-        self.selection = self.selection.extend(other.selection);
-        self.input_and_logs = self.input_and_logs.extend(other.input_and_logs);
-        self.help_menu = self.help_menu.extend(other.help_menu);
-        self
-    }
-
     /// Get a reference to the panel ui's default.
     pub fn default(&self) -> &PanelUiConfig {
         &self.default
@@ -504,67 +431,49 @@ impl PanelUi {
 #[serde(deny_unknown_fields)]
 pub struct GeneralConfig {
     #[serde(default)]
-    show_hidden: Option<bool>,
+    pub show_hidden: Option<bool>,
 
     #[serde(default)]
-    read_only: Option<bool>,
+    pub read_only: Option<bool>,
 
     #[serde(default)]
-    cursor: UiElement,
+    pub cursor: UiElement,
 
     #[serde(default)]
-    prompt: UiElement,
+    pub prompt: UiElement,
 
     #[serde(default)]
-    logs: LogsConfig,
+    pub logs: LogsConfig,
 
     #[serde(default)]
-    table: TableConfig,
+    pub table: TableConfig,
 
     #[serde(default)]
-    default_ui: UiConfig,
+    pub default_ui: UiConfig,
 
     #[serde(default)]
-    focus_ui: UiConfig,
+    pub focus_ui: UiConfig,
 
     #[serde(default)]
-    selection_ui: UiConfig,
+    pub selection_ui: UiConfig,
 
     #[serde(default)]
-    sort_and_filter_ui: SortAndFilterUi,
+    pub sort_and_filter_ui: SortAndFilterUi,
 
     #[serde(default)]
-    panel_ui: PanelUi,
+    pub panel_ui: PanelUi,
 
     #[serde(default)]
-    initial_sorting: Option<IndexSet<NodeSorterApplicable>>,
+    pub initial_sorting: Option<IndexSet<NodeSorterApplicable>>,
 
     #[serde(default)]
-    initial_mode: Option<String>,
+    pub initial_mode: Option<String>,
 
     #[serde(default)]
-    initial_layout: Option<String>,
+    pub initial_layout: Option<String>,
 }
 
 impl GeneralConfig {
-    fn extend(mut self, other: Self) -> Self {
-        self.show_hidden = other.show_hidden.or(self.show_hidden);
-        self.read_only = other.read_only.or(self.read_only);
-        self.cursor = self.cursor.extend(other.cursor);
-        self.prompt = self.prompt.extend(other.prompt);
-        self.logs = self.logs.extend(other.logs);
-        self.table = self.table.extend(other.table);
-        self.default_ui = self.default_ui.extend(other.default_ui);
-        self.focus_ui = self.focus_ui.extend(other.focus_ui);
-        self.selection_ui = self.selection_ui.extend(other.selection_ui);
-        self.sort_and_filter_ui = self.sort_and_filter_ui.extend(other.sort_and_filter_ui);
-        self.panel_ui = self.panel_ui.extend(other.panel_ui);
-        self.initial_sorting = other.initial_sorting.or(self.initial_sorting);
-        self.initial_layout = other.initial_layout.or(self.initial_layout);
-        self.initial_mode = other.initial_mode.or(self.initial_mode);
-        self
-    }
-
     /// Get a reference to the general config's show hidden.
     pub fn show_hidden(&self) -> Option<bool> {
         self.show_hidden
@@ -636,26 +545,26 @@ impl GeneralConfig {
     }
 }
 
-#[derive(Debug, Clone, Default, Serialize, Deserialize)]
+#[derive(Debug, Clone, Default, PartialEq, Serialize, Deserialize)]
 #[serde(deny_unknown_fields)]
 pub struct KeyBindings {
     #[serde(default)]
-    remaps: BTreeMap<String, String>,
+    pub remaps: IndexMap<String, Option<String>>,
 
     #[serde(default)]
-    on_key: BTreeMap<String, Action>,
+    pub on_key: BTreeMap<String, Action>,
 
     #[serde(default)]
-    on_alphabet: Option<Action>,
+    pub on_alphabet: Option<Action>,
 
     #[serde(default)]
-    on_number: Option<Action>,
+    pub on_number: Option<Action>,
 
     #[serde(default)]
-    on_special_character: Option<Action>,
+    pub on_special_character: Option<Action>,
 
     #[serde(default)]
-    default: Option<Action>,
+    pub default: Option<Action>,
 }
 
 impl KeyBindings {
@@ -673,28 +582,27 @@ impl KeyBindings {
                 .on_special_character
                 .and_then(|a| a.sanitized(read_only));
             self.default = self.default.and_then(|a| a.sanitized(read_only));
-            self.remaps = self
-                .remaps
-                .clone()
-                .into_iter()
-                .filter(|(_, v)| self.on_key.contains_key(v))
-                .collect();
         };
-        self
-    }
 
-    fn extend(mut self, other: Self) -> Self {
-        self.remaps.extend(other.remaps);
-        self.on_key.extend(other.on_key);
-        self.on_alphabet = other.on_alphabet.or(self.on_alphabet);
-        self.on_number = other.on_number.or(self.on_number);
-        self.on_special_character = other.on_special_character.or(self.on_special_character);
-        self.default = other.default.or(self.default);
+        let mut remaps = IndexMap::new();
+        for (from, maybe_to) in self.remaps.into_iter() {
+            if let Some(to) = maybe_to.as_ref() {
+                let mapped = self.on_key.get(to).cloned();
+                if let Some(a) = mapped {
+                    self.on_key.insert(from.clone(), a.clone());
+                    remaps.insert(from, maybe_to);
+                }
+            } else {
+                self.on_key.remove(&from);
+            }
+        }
+
+        self.remaps = remaps;
         self
     }
 
     /// Get a reference to the key bindings's remaps.
-    pub fn remaps(&self) -> &BTreeMap<String, String> {
+    pub fn remaps(&self) -> &IndexMap<String, Option<String>> {
         &self.remaps
     }
 
@@ -724,20 +632,20 @@ impl KeyBindings {
     }
 }
 
-#[derive(Debug, Clone, Default, Serialize, Deserialize)]
+#[derive(Debug, Clone, Default, PartialEq, Serialize, Deserialize)]
 #[serde(deny_unknown_fields)]
 pub struct Mode {
     #[serde(default)]
-    name: String,
+    pub name: String,
 
     #[serde(default)]
-    help: Option<String>,
+    pub help: Option<String>,
 
     #[serde(default)]
-    extra_help: Option<String>,
+    pub extra_help: Option<String>,
 
     #[serde(default)]
-    key_bindings: KeyBindings,
+    pub key_bindings: KeyBindings,
 }
 
 impl Mode {
@@ -746,13 +654,6 @@ impl Mode {
         self
     }
 
-    fn extend(mut self, other: Self) -> Self {
-        self.help = other.help.or(self.help);
-        self.extra_help = other.extra_help.or(self.extra_help);
-        self.key_bindings = self.key_bindings.extend(other.key_bindings);
-        self
-    }
-
     pub fn help_menu(&self) -> Vec<HelpMenuLine> {
         let extra_help_lines = self.extra_help.clone().map(|e| {
             e.lines()
@@ -775,7 +676,17 @@ impl Mode {
                         self.key_bindings
                             .on_key
                             .iter()
-                            .filter(|(k, _)| !self.key_bindings.remaps.contains_key(&k.to_string()))
+                            .filter(|(k, v)| {
+                                !self
+                                    .key_bindings
+                                    .remaps
+                                    .get(*k)
+                                    .and_then(|mt| {
+                                        mt.as_ref()
+                                            .map(|t| self.key_bindings.on_key.get(t) == Some(v))
+                                    })
+                                    .unwrap_or(false)
+                            })
                             .filter_map(|(k, a)| {
                                 a.help.clone().map(|h| HelpMenuLine::KeyMap(k.into(), h))
                             }),
@@ -837,82 +748,62 @@ impl Mode {
 #[serde(deny_unknown_fields)]
 pub struct BuiltinModesConfig {
     #[serde(default)]
-    default: Mode,
+    pub default: Mode,
 
     #[serde(default)]
-    selection_ops: Mode,
+    pub recover: Mode,
 
     #[serde(default)]
-    create: Mode,
+    pub selection_ops: Mode,
 
     #[serde(default)]
-    create_directory: Mode,
+    pub create: Mode,
 
     #[serde(default)]
-    create_file: Mode,
+    pub create_directory: Mode,
 
     #[serde(default)]
-    number: Mode,
+    pub create_file: Mode,
 
     #[serde(default)]
-    go_to: Mode,
+    pub number: Mode,
 
     #[serde(default)]
-    rename: Mode,
+    pub go_to: Mode,
 
     #[serde(default)]
-    delete: Mode,
+    pub rename: Mode,
 
     #[serde(default)]
-    action: Mode,
+    pub delete: Mode,
 
     #[serde(default)]
-    search: Mode,
+    pub action: Mode,
 
     #[serde(default)]
-    filter: Mode,
+    pub search: Mode,
 
     #[serde(default)]
-    relative_path_does_contain: Mode,
+    pub filter: Mode,
 
     #[serde(default)]
-    relative_path_does_not_contain: Mode,
+    pub relative_path_does_contain: Mode,
 
     #[serde(default)]
-    sort: Mode,
+    pub relative_path_does_not_contain: Mode,
 
     #[serde(default)]
-    switch_layout: Mode,
+    pub sort: Mode,
+
+    #[serde(default)]
+    pub switch_layout: Mode,
 }
 
 impl BuiltinModesConfig {
-    pub fn extend(mut self, other: Self) -> Self {
-        self.default = self.default.extend(other.default);
-        self.selection_ops = self.selection_ops.extend(other.selection_ops);
-        self.go_to = self.go_to.extend(other.go_to);
-        self.create = self.create.extend(other.create);
-        self.create_file = self.create_file.extend(other.create_file);
-        self.create_directory = self.create_directory.extend(other.create_directory);
-        self.rename = self.rename.extend(other.rename);
-        self.delete = self.delete.extend(other.delete);
-        self.number = self.number.extend(other.number);
-        self.action = self.action.extend(other.action);
-        self.search = self.search.extend(other.search);
-        self.filter = self.filter.extend(other.filter);
-        self.relative_path_does_contain = self
-            .relative_path_does_contain
-            .extend(other.relative_path_does_contain);
-        self.relative_path_does_not_contain = self
-            .relative_path_does_not_contain
-            .extend(other.relative_path_does_not_contain);
-        self.sort = self.sort.extend(other.sort);
-        self.switch_layout = self.switch_layout.extend(other.switch_layout);
-        self
-    }
-
     pub fn get(&self, name: &str) -> Option<&Mode> {
         match name {
             "default" => Some(&self.default),
+            "recover" => Some(&self.recover),
             "selection ops" => Some(&self.selection_ops),
             "selection_ops" => Some(&self.selection_ops),
             "create" => Some(&self.create),
@@ -1018,16 +909,21 @@ impl BuiltinModesConfig {
     pub fn switch_layout(&self) -> &Mode {
         &self.switch_layout
     }
+
+    /// Get a reference to the builtin modes config's recover.
+    pub fn recover(&self) -> &Mode {
+        &self.recover
+    }
 }
 
 #[derive(Debug, Clone, Default, Serialize, Deserialize)]
 #[serde(deny_unknown_fields)]
 pub struct ModesConfig {
     #[serde(default)]
-    builtin: BuiltinModesConfig,
+    pub builtin: BuiltinModesConfig,
 
     #[serde(default)]
-    custom: HashMap<String, Mode>,
+    pub custom: HashMap<String, Mode>,
 }
 
 impl ModesConfig {
@@ -1043,12 +939,6 @@ impl ModesConfig {
         self.get_builtin(name).or_else(|| self.get_custom(name))
     }
 
-    pub fn extend(mut self, other: Self) -> Self {
-        self.builtin = self.builtin.extend(other.builtin);
-        self.custom.extend(other.custom);
-        self
-    }
-
     /// Get a reference to the modes config's builtin.
     pub fn builtin(&self) -> &BuiltinModesConfig {
         &self.builtin
@@ -1064,13 +954,13 @@ impl ModesConfig {
 #[serde(deny_unknown_fields)]
 pub struct PanelUiConfig {
     #[serde(default)]
-    title: UiElement,
+    pub title: UiElement,
 
     #[serde(default)]
-    borders: Option<IndexSet<Border>>,
+    pub borders: Option<IndexSet<Border>>,
 
     #[serde(default)]
-    style: Style,
+    pub style: Style,
 }
 
 impl PanelUiConfig {
@@ -1100,27 +990,19 @@ impl PanelUiConfig {
 #[serde(deny_unknown_fields)]
 pub struct BuiltinLayoutsConfig {
     #[serde(default)]
-    default: Layout,
+    pub default: Layout,
 
     #[serde(default)]
-    no_help: Layout,
+    pub no_help: Layout,
 
     #[serde(default)]
-    no_selection: Layout,
+    pub no_selection: Layout,
 
     #[serde(default)]
-    no_help_no_selection: Layout,
+    pub no_help_no_selection: Layout,
 }
 
 impl BuiltinLayoutsConfig {
-    pub fn extend(mut self, other: Self) -> Self {
-        self.default = self.default.extend(other.default);
-        self.no_help = self.no_help.extend(other.no_help);
-        self.no_selection = self.no_selection.extend(other.no_selection);
-        self.no_help_no_selection = self.no_help_no_selection.extend(other.no_help_no_selection);
-        self
-    }
-
     pub fn get(&self, name: &str) -> Option<&Layout> {
         match name {
             "default" => Some(&self.default),
@@ -1144,10 +1026,10 @@ impl BuiltinLayoutsConfig {
 #[serde(deny_unknown_fields)]
 pub struct LayoutsConfig {
     #[serde(default)]
-    builtin: BuiltinLayoutsConfig,
+    pub builtin: BuiltinLayoutsConfig,
 
     #[serde(default)]
-    custom: HashMap<String, Layout>,
+    pub custom: HashMap<String, Layout>,
 }
 
 impl LayoutsConfig {
@@ -1163,12 +1045,6 @@ impl LayoutsConfig {
         self.get_builtin(name).or_else(|| self.get_custom(name))
     }
 
-    pub fn extend(mut self, other: Self) -> Self {
-        self.builtin = self.builtin.extend(other.builtin);
-        self.custom.extend(other.custom);
-        self
-    }
-
     /// Get a reference to the layouts config's builtin.
     pub fn builtin(&self) -> &BuiltinLayoutsConfig {
         &self.builtin
@@ -1180,93 +1056,23 @@ impl LayoutsConfig {
     }
 }
 
-#[derive(Debug, Clone, Serialize, Deserialize)]
+#[derive(Debug, Default, Clone, Serialize, Deserialize)]
 #[serde(deny_unknown_fields)]
 pub struct Config {
-    version: String,
+    #[serde(default)]
+    pub layouts: LayoutsConfig,
 
     #[serde(default)]
-    layouts: LayoutsConfig,
+    pub general: GeneralConfig,
 
     #[serde(default)]
-    general: GeneralConfig,
+    pub node_types: NodeTypesConfig,
 
     #[serde(default)]
-    node_types: NodeTypesConfig,
-
-    #[serde(default)]
-    modes: ModesConfig,
-}
-
-impl Default for Config {
-    fn default() -> Self {
-        Self {
-            version: default_config::version(),
-            layouts: default_config::layouts(),
-            general: default_config::general(),
-            node_types: default_config::node_types(),
-            modes: default_config::modes(),
-        }
-    }
+    pub modes: ModesConfig,
 }
 
 impl Config {
-    pub fn extended(mut self) -> Self {
-        let default = Self::default();
-        self.layouts = default.layouts.extend(self.layouts);
-        self.general = default.general.extend(self.general);
-        self.node_types = default.node_types.extend(self.node_types);
-        self.modes = default.modes.extend(self.modes);
-        self
-    }
-
-    fn parsed_version(&self) -> Result<(u16, u16, u16)> {
-        let mut configv = self
-            .version
-            .strip_prefix('v')
-            .unwrap_or_default()
-            .split('.');
-
-        let major = configv.next().unwrap_or_default().parse::<u16>()?;
-        let minor = configv.next().unwrap_or_default().parse::<u16>()?;
-        let bugfix = configv.next().unwrap_or_default().parse::<u16>()?;
-
-        Ok((major, minor, bugfix))
-    }
-
-    pub fn is_compatible(&self) -> Result<bool> {
-        let result = match self.parsed_version()? {
-            (0, 8, 5) => true,
-            (0, 8, 4) => true,
-            (0, 8, 3) => true,
-            (0, 8, 2) => true,
-            (0, 8, 1) => true,
-            (0, 8, 0) => true,
-            (_, _, _) => false,
-        };
-
-        Ok(result)
-    }
-
-    pub fn upgrade_notification(&self) -> Result<Option<&str>> {
-        let result = match self.parsed_version()? {
-            (0, 8, 5) => None,
-            (0, 8, 4) => Some("App version updated. Improved key bindings"),
-            (0, 8, 3) => Some("App version updated. Fixed search"),
-            (0, 8, 2) => Some("App version updated. Fixed search"),
-            (0, 8, 1) => Some("App version updated. Fixed search"),
-            (0, 8, 0) => Some("App version updated. Fixed breaking UI after rename"),
-            (_, _, _) => None,
-        };
-
-        Ok(result)
-    }
-
-    /// Get a reference to the config's version.
-    pub fn version(&self) -> &String {
-        &self.version
-    }
-
     /// Get a reference to the config's layouts.
     pub fn layouts(&self) -> &LayoutsConfig {
         &self.layouts
@@ -1287,35 +1093,3 @@ impl Config {
         &self.modes
     }
 }
-
-#[cfg(test)]
-mod test {
-
-    use super::*;
-    use std::collections::HashMap;
-
-    #[test]
-    fn test_compatibility() {
-        let config = Config::default();
-        assert!(config.is_compatible().unwrap());
-        assert_eq!(config.upgrade_notification().unwrap(), None);
-    }
-
-    #[test]
-    fn test_extend_hashmap() {
-        let mut a = HashMap::new();
-        let mut b = HashMap::new();
-
-        a.insert("a", "a");
-        a.insert("b", "a");
-
-        b.insert("b", "b");
-        b.insert("c", "b");
-
-        a.extend(b);
-
-        assert_eq!(a.get("a"), Some(&"a"));
-        assert_eq!(a.get("b"), Some(&"b"));
-        assert_eq!(a.get("c"), Some(&"b"));
-    }
-}
