@@ -5,10 +5,10 @@
 pub mod app;
 pub mod auto_refresher;
 pub mod config;
-pub mod default_config;
 pub mod event_reader;
 pub mod explorer;
 pub mod input;
+pub mod lua;
 pub mod pipe_reader;
 pub mod pwd_watcher;
 pub mod runner;
