@@ -2,9 +2,11 @@ use crate::app;
 use crate::app::HelpMenuLine;
 use crate::app::{Node, ResolvedNode};
 use crate::config::PanelUiConfig;
-use handlebars::Handlebars;
+use crate::lua;
 use indexmap::IndexSet;
 use lazy_static::lazy_static;
+use mlua::Lua;
+use mlua::LuaSerdeExt;
 use serde::{Deserialize, Serialize};
 use std::cmp::Ordering;
 use std::collections::HashMap;
@@ -26,13 +28,16 @@ lazy_static! {
 #[serde(deny_unknown_fields)]
 pub struct LayoutOptions {
     #[serde(default)]
-    margin: Option<u16>,
+    pub margin: Option<u16>,
+
     #[serde(default)]
-    horizontal_margin: Option<u16>,
+    pub horizontal_margin: Option<u16>,
+
     #[serde(default)]
-    vertical_margin: Option<u16>,
+    pub vertical_margin: Option<u16>,
+
     #[serde(default)]
-    constraints: Option<Vec<Constraint>>,
+    pub constraints: Option<Vec<Constraint>>,
 }
 
 impl LayoutOptions {
@@ -68,12 +73,12 @@ impl LayoutOptions {
 #[derive(Debug, Clone, Serialize, Deserialize)]
 #[serde(deny_unknown_fields)]
 pub enum Layout {
-    Nothing(Option<PanelUiConfig>),
-    Table(Option<PanelUiConfig>),
-    InputAndLogs(Option<PanelUiConfig>),
-    Selection(Option<PanelUiConfig>),
-    HelpMenu(Option<PanelUiConfig>),
-    SortAndFilter(Option<PanelUiConfig>),
+    Nothing,
+    Table,
+    InputAndLogs,
+    Selection,
+    HelpMenu,
+    SortAndFilter,
     Horizontal {
         config: LayoutOptions,
         splits: Vec<Layout>,
@@ -86,19 +91,14 @@ pub enum Layout {
 
 impl Default for Layout {
     fn default() -> Self {
-        Self::Nothing(Default::default())
+        Self::Nothing
     }
 }
 
 impl Layout {
     pub fn extend(self, other: Self) -> Self {
         match (self, other) {
-            (s, Self::Nothing(_)) => s,
-            (Self::Table(s), Self::Table(o)) => Self::Table(o.or(s)),
-            (Self::InputAndLogs(s), Self::InputAndLogs(o)) => Self::InputAndLogs(o.or(s)),
-            (Self::Selection(s), Self::Selection(o)) => Self::Selection(o.or(s)),
-            (Self::HelpMenu(s), Self::HelpMenu(o)) => Self::HelpMenu(o.or(s)),
-            (Self::SortAndFilter(s), Self::SortAndFilter(o)) => Self::SortAndFilter(o.or(s)),
+            (s, Self::Nothing) => s,
             (
                 Self::Horizontal {
                     config: sconfig,
@@ -184,10 +184,10 @@ impl Modifier {
 #[derive(Debug, Clone, Default, Serialize, Deserialize)]
 #[serde(deny_unknown_fields)]
 pub struct Style {
-    fg: Option<Color>,
-    bg: Option<Color>,
-    add_modifiers: Option<IndexSet<Modifier>>,
-    sub_modifiers: Option<IndexSet<Modifier>>,
+    pub fg: Option<Color>,
+    pub bg: Option<Color>,
+    pub add_modifiers: Option<IndexSet<Modifier>>,
+    pub sub_modifiers: Option<IndexSet<Modifier>>,
 }
 
 impl PartialEq for Style {
@@ -292,15 +292,15 @@ impl Constraint {
 }
 
 #[derive(Debug, Clone, Serialize, Deserialize)]
-#[serde(rename_all = "camelCase")]
 pub struct ResolvedNodeUiMetadata {
-    absolute_path: String,
-    extension: String,
-    is_dir: bool,
-    is_file: bool,
-    is_readonly: bool,
-    mime_essence: String,
-    size: u64,
+    pub absolute_path: String,
+    pub extension: String,
+    pub is_dir: bool,
+    pub is_file: bool,
+    pub is_readonly: bool,
+    pub mime_essence: String,
+    pub size: u64,
+    pub human_size: String,
 }
 
 impl From<ResolvedNode> for ResolvedNodeUiMetadata {
@@ -313,40 +313,41 @@ impl From<ResolvedNode> for ResolvedNodeUiMetadata {
             is_readonly: node.is_readonly(),
             mime_essence: node.mime_essence().clone(),
             size: node.size(),
+            human_size: node.human_size().clone(),
         }
     }
 }
 
 #[derive(Debug, Clone, Serialize, Deserialize)]
-#[serde(rename_all = "camelCase")]
 struct NodeUiMetadata {
     // From Node
-    parent: String,
-    relative_path: String,
-    absolute_path: String,
-    extension: String,
-    is_symlink: bool,
-    is_broken: bool,
-    is_dir: bool,
-    is_file: bool,
-    is_readonly: bool,
-    mime_essence: String,
-    size: u64,
-    canonical: Option<ResolvedNodeUiMetadata>,
-    symlink: Option<ResolvedNodeUiMetadata>,
+    pub parent: String,
+    pub relative_path: String,
+    pub absolute_path: String,
+    pub extension: String,
+    pub is_symlink: bool,
+    pub is_broken: bool,
+    pub is_dir: bool,
+    pub is_file: bool,
+    pub is_readonly: bool,
+    pub mime_essence: String,
+    pub size: u64,
+    pub human_size: String,
+    pub canonical: Option<ResolvedNodeUiMetadata>,
+    pub symlink: Option<ResolvedNodeUiMetadata>,
 
     // Extra
-    index: usize,
-    relative_index: usize,
-    is_before_focus: bool,
-    is_after_focus: bool,
-    tree: String,
-    prefix: String,
-    suffix: String,
-    is_selected: bool,
-    is_focused: bool,
-    total: usize,
-    meta: HashMap<String, String>,
+    pub index: usize,
+    pub relative_index: usize,
+    pub is_before_focus: bool,
+    pub is_after_focus: bool,
+    pub tree: String,
+    pub prefix: String,
+    pub suffix: String,
+    pub is_selected: bool,
+    pub is_focused: bool,
+    pub total: usize,
+    pub meta: HashMap<String, String>,
 }
 
 impl NodeUiMetadata {
@@ -376,6 +377,7 @@ impl NodeUiMetadata {
             is_readonly: node.is_readonly(),
             mime_essence: node.mime_essence().clone(),
             size: node.size(),
+            human_size: node.human_size().clone(),
             canonical: node.canonical().to_owned().map(|s| s.into()),
             symlink: node.symlink().to_owned().map(|s| s.into()),
             index,
@@ -412,21 +414,17 @@ fn block<'a>(config: PanelUiConfig, default_title: String) -> Block<'a> {
 }
 
 fn draw_table<B: Backend>(
-    config: Option<PanelUiConfig>,
     f: &mut Frame<B>,
     screen_size: Rect,
     layout_size: Rect,
     app: &app::App,
-    hb: &Handlebars,
+    lua: &Lua,
 ) {
     let panel_config = app.config().general().panel_ui();
-    let default_panel_config = panel_config
+    let config = panel_config
         .default()
         .clone()
         .extend(panel_config.table().clone());
-    let config = config
-        .map(|c| default_panel_config.clone().extend(c))
-        .unwrap_or(default_panel_config);
     let app_config = app.config().to_owned();
     let header_height = app_config.general().table().header().height().unwrap_or(1);
     let height: usize = (layout_size.height.max(header_height + 2) - (header_height + 2)).into();
@@ -530,12 +528,27 @@ fn draw_table<B: Backend>(
                         node_type.meta().clone(),
                     );
 
-                    let cols = hb
-                        .render(app::TEMPLATE_TABLE_ROW, &meta)
-                        .ok()
-                        .unwrap_or_else(|| app::UNSUPPORTED_STR.into())
-                        .split('\t')
-                        .map(|x| Cell::from(x.to_string()))
+                    let cols = lua
+                        .to_value::<NodeUiMetadata>(&meta)
+                        .map(|v| {
+                            app_config
+                                .general()
+                                .table()
+                                .row()
+                                .cols()
+                                .clone()
+                                .unwrap_or_default()
+                                .iter()
+                                .filter_map(|c| {
+                                    c.format().as_ref().map(|f| {
+                                        lua::call(lua, f, &v).unwrap_or_else(|e| e.to_string())
+                                    })
+                                })
+                                .collect::<Vec<String>>()
+                        })
+                        .unwrap_or_default()
+                        .iter()
+                        .map(|x| Cell::from(x.to_owned()))
                         .collect::<Vec<Cell>>();
 
                     Row::new(cols).style(style.into())
@@ -597,21 +610,17 @@ fn draw_table<B: Backend>(
 }
 
 fn draw_selection<B: Backend>(
-    config: Option<PanelUiConfig>,
     f: &mut Frame<B>,
     _screen_size: Rect,
     layout_size: Rect,
     app: &app::App,
-    _: &Handlebars,
+    _: &Lua,
 ) {
     let panel_config = app.config().general().panel_ui();
-    let default_panel_config = panel_config
+    let config = panel_config
         .default()
         .clone()
         .extend(panel_config.selection().clone());
-    let config = config
-        .map(|c| default_panel_config.clone().extend(c))
-        .unwrap_or(default_panel_config);
     let selection: Vec<ListItem> = app
         .selection()
         .iter()
@@ -632,21 +641,17 @@ fn draw_selection<B: Backend>(
 }
 
 fn draw_help_menu<B: Backend>(
-    config: Option<PanelUiConfig>,
     f: &mut Frame<B>,
     _screen_size: Rect,
     layout_size: Rect,
     app: &app::App,
-    _: &Handlebars,
+    _: &Lua,
 ) {
     let panel_config = app.config().general().panel_ui();
-    let default_panel_config = panel_config
+    let config = panel_config
         .default()
         .clone()
         .extend(panel_config.help_menu().clone());
-    let config = config
-        .map(|c| default_panel_config.clone().extend(c))
-        .unwrap_or(default_panel_config);
     let help_menu_rows = app
         .mode()
         .help_menu()
@@ -688,21 +693,17 @@ fn draw_help_menu<B: Backend>(
 }
 
 fn draw_input_buffer<B: Backend>(
-    config: Option<PanelUiConfig>,
     f: &mut Frame<B>,
     _screen_size: Rect,
     layout_size: Rect,
     app: &app::App,
-    _: &Handlebars,
+    _: &Lua,
 ) {
     let panel_config = app.config().general().panel_ui();
-    let default_panel_config = panel_config
+    let config = panel_config
         .default()
         .clone()
         .extend(panel_config.input_and_logs().clone());
-    let config = config
-        .map(|c| default_panel_config.clone().extend(c))
-        .unwrap_or(default_panel_config);
     let input_buf = Paragraph::new(Spans::from(vec![
         Span::styled(
             app.config()
@@ -729,21 +730,17 @@ fn draw_input_buffer<B: Backend>(
 }
 
 fn draw_sort_n_filter<B: Backend>(
-    config: Option<PanelUiConfig>,
     f: &mut Frame<B>,
     _screen_size: Rect,
     layout_size: Rect,
     app: &app::App,
-    _: &Handlebars,
+    _: &Lua,
 ) {
     let panel_config = app.config().general().panel_ui();
-    let default_panel_config = panel_config
+    let config = panel_config
         .default()
         .clone()
         .extend(panel_config.sort_and_filter().clone());
-    let config = config
-        .map(|c| default_panel_config.clone().extend(c))
-        .unwrap_or(default_panel_config);
     let ui = app.config().general().sort_and_filter_ui().clone();
     let filter_by = app.explorer_config().filters();
     let sort_by = app.explorer_config().sorters();
@@ -810,21 +807,17 @@ fn draw_sort_n_filter<B: Backend>(
 }
 
 fn draw_logs<B: Backend>(
-    config: Option<PanelUiConfig>,
     f: &mut Frame<B>,
     _screen_size: Rect,
     layout_size: Rect,
     app: &app::App,
-    _: &Handlebars,
+    _: &Lua,
 ) {
     let panel_config = app.config().general().panel_ui();
-    let default_panel_config = panel_config
+    let config = panel_config
         .default()
         .clone()
         .extend(panel_config.input_and_logs().clone());
-    let config = config
-        .map(|c| default_panel_config.clone().extend(c))
-        .unwrap_or(default_panel_config);
     let logs_config = app.config().general().logs().clone();
     let logs = if app.logs_hidden() {
         vec![]
@@ -887,18 +880,14 @@ fn draw_logs<B: Backend>(
 }
 
 pub fn draw_nothing<B: Backend>(
-    config: Option<PanelUiConfig>,
     f: &mut Frame<B>,
     _screen_size: Rect,
     layout_size: Rect,
     app: &app::App,
-    _hb: &Handlebars,
+    _lua: &Lua,
 ) {
     let panel_config = app.config().general().panel_ui();
-    let default_panel_config = panel_config.default().clone();
-    let config = config
-        .map(|c| default_panel_config.clone().extend(c))
-        .unwrap_or(default_panel_config);
+    let config = panel_config.default().clone();
     let nothing = Paragraph::new("").block(block(config, "".into()));
     f.render_widget(nothing, layout_size);
 }
@@ -909,21 +898,19 @@ pub fn draw_layout<B: Backend>(
     screen_size: Rect,
     layout_size: Rect,
     app: &app::App,
-    hb: &Handlebars,
+    lua: &Lua,
 ) {
     match layout {
-        Layout::Nothing(config) => draw_nothing(config, f, screen_size, layout_size, app, hb),
-        Layout::Table(config) => draw_table(config, f, screen_size, layout_size, app, hb),
-        Layout::SortAndFilter(config) => {
-            draw_sort_n_filter(config, f, screen_size, layout_size, app, hb)
-        }
-        Layout::HelpMenu(config) => draw_help_menu(config, f, screen_size, layout_size, app, hb),
-        Layout::Selection(config) => draw_selection(config, f, screen_size, layout_size, app, hb),
-        Layout::InputAndLogs(config) => {
+        Layout::Nothing => draw_nothing(f, screen_size, layout_size, app, lua),
+        Layout::Table => draw_table(f, screen_size, layout_size, app, lua),
+        Layout::SortAndFilter => draw_sort_n_filter(f, screen_size, layout_size, app, lua),
+        Layout::HelpMenu => draw_help_menu(f, screen_size, layout_size, app, lua),
+        Layout::Selection => draw_selection(f, screen_size, layout_size, app, lua),
+        Layout::InputAndLogs => {
             if app.input_buffer().is_some() {
-                draw_input_buffer(config, f, screen_size, layout_size, app, hb);
+                draw_input_buffer(f, screen_size, layout_size, app, lua);
             } else {
-                draw_logs(config, f, screen_size, layout_size, app, hb);
+                draw_logs(f, screen_size, layout_size, app, lua);
             };
         }
         Layout::Horizontal { config, splits } => {
@@ -955,7 +942,7 @@ pub fn draw_layout<B: Backend>(
             splits
                 .into_iter()
                 .zip(chunks.into_iter())
-                .for_each(|(split, chunk)| draw_layout(split, f, screen_size, chunk, app, hb));
+                .for_each(|(split, chunk)| draw_layout(split, f, screen_size, chunk, app, lua));
         }
 
         Layout::Vertical { config, splits } => {
@@ -987,22 +974,21 @@ pub fn draw_layout<B: Backend>(
             splits
                 .into_iter()
                 .zip(chunks.into_iter())
-                .for_each(|(split, chunk)| draw_layout(split, f, screen_size, chunk, app, hb));
+                .for_each(|(split, chunk)| draw_layout(split, f, screen_size, chunk, app, lua));
         }
     }
 }
 
-pub fn draw<B: Backend>(f: &mut Frame<B>, app: &app::App, hb: &Handlebars) {
+pub fn draw<B: Backend>(f: &mut Frame<B>, app: &app::App, lua: &Lua) {
     let screen_size = f.size();
     let layout = app.layout().clone();
 
-    draw_layout(layout, f, screen_size, screen_size, app, hb);
+    draw_layout(layout, f, screen_size, screen_size, app, lua);
 }
 
 #[cfg(test)]
 mod test {
     use super::*;
-    use crate::config;
     use tui::style::Color;
 
     fn modifier(m: Modifier) -> Option<IndexSet<Modifier>> {
@@ -1074,82 +1060,4 @@ mod test {
             }
         );
     }
-
-    #[test]
-    fn test_extend_ui_config() {
-        let a = config::UiConfig {
-            prefix: Some("a".to_string()),
-            suffix: None,
-            style: Style {
-                fg: Some(Color::Red),
-                bg: None,
-                add_modifiers: modifier(Modifier::Bold),
-                sub_modifiers: None,
-            },
-        };
-
-        let b = config::UiConfig {
-            prefix: None,
-            suffix: Some("b".to_string()),
-            style: Style {
-                fg: None,
-                bg: Some(Color::Blue),
-                add_modifiers: None,
-                sub_modifiers: modifier(Modifier::Dim),
-            },
-        };
-
-        let c = config::UiConfig {
-            prefix: Some("cp".to_string()),
-            suffix: Some("cs".to_string()),
-            style: Style {
-                fg: Some(Color::Cyan),
-                bg: Some(Color::Magenta),
-                add_modifiers: modifier(Modifier::CrossedOut),
-                sub_modifiers: modifier(Modifier::Italic),
-            },
-        };
-
-        assert_eq!(
-            a.clone().extend(b.clone()),
-            config::UiConfig {
-                prefix: Some("a".to_string()),
-                suffix: Some("b".to_string()),
-                style: Style {
-                    fg: Some(Color::Red),
-                    bg: Some(Color::Blue),
-                    add_modifiers: modifier(Modifier::Bold),
-                    sub_modifiers: modifier(Modifier::Dim),
-                },
-            }
-        );
-
-        assert_eq!(
-            b.clone().extend(a.clone()),
-            config::UiConfig {
-                prefix: Some("a".to_string()),
-                suffix: Some("b".to_string()),
-                style: Style {
-                    fg: Some(Color::Red),
-                    bg: Some(Color::Blue),
-                    add_modifiers: modifier(Modifier::Bold),
-                    sub_modifiers: modifier(Modifier::Dim),
-                },
-            }
-        );
-
-        assert_eq!(
-            a.clone().extend(c.clone()),
-            config::UiConfig {
-                prefix: Some("cp".to_string()),
-                suffix: Some("cs".to_string()),
-                style: Style {
-                    fg: Some(Color::Cyan),
-                    bg: Some(Color::Magenta),
-                    add_modifiers: modifier(Modifier::CrossedOut),
-                    sub_modifiers: modifier(Modifier::Italic),
-                },
-            }
-        );
-    }
 }
